#!/bin/bash
############################################################################
# Group Trial Script 0.5 by Turranius                                      #
# This script will check how much users in a group uploaded this week and  #
# add it togheter. It will then check the passlimit to see if they passed  #
# the trial week.                                                          #
#                                                                          #
# Installation:                                                            #
# Copy this .sh script to /glftpd/bin and make sure the user who runs the  #
# bot has execute permissions on it.                                       #
# Copy the .tcl to your bots /scripts folder and load it in the bots config#
# file. Rehash the bot. Note that this is only needed IF you want to       #
# trigger it from IRC as well.                                             #
# Add this .sh to glftpd.conf as a custom command. Preferebly like:        #
# site_cmd GTRIAL         EXEC    /bin/trialgroup.sh                       #
# custom-gtrial           *                                                #
# Note that this is only needed IF you wish to execute it from within gl.  #
#                                                                          #
# Edit the paths and passlimit below to suit your needs.                   #
# Edit the .tcl script to perhaps change the trigger.                      #
#                                                                          #
# Now then, if you are running this outside of glftpds chrooted enviroment #
# add a second command called "shell".                                     #
# Example: grouptrial.sh iND shell                                         #
# Note: This is already in the .tcl. To check a group from IRC, simply do  #
# !gtrial <group>                                                          #
#                                                                          # 
# It will not delete any users or do anything else. It will simply report  #
# how well the group have done this week. The rest is up to you.           #
#                                                                          #
# Also check out the trialgroup.cron.sh script. It will do the same thing  #
# but in scheduled cron job to a log file so you dont have to run this one #
# at 23:59 on saturday.                                                    #
# Note, since 0.4, you can just put this script in a crontab and pipe it   #
# to a file instead. Dont forget the "shell" part when executing it.       #
############################################################################
# Changelog                                                                #
# 0.1 - Initial script.                                                    #
# 0.2 - Added TRiAL-2 which is to be used if they are up for trial next    #
#       week. Have to change it to TRiAL manually when its time for them   #
#       to start.                                                          #
#       Changed how the script checks if they are on trial. Instead of     #
#       reading a file, it checks the groups NFO. To set groups on trial   #
#       change the groups nfo (with site grpnfo) to either TRiAL or        #
#       TRiAL-2.                                                           #
# 0.3 - Just changed so announce to irc is one line instead of two.        #
# 0.4 - Removed the writing to glftpd.log, and instead added a .tcl to     #
#       execute it from IRC.                                               #
#       Read new installation instructions.                                #
# 0.5 - Add: Handles up to 9 stat sections automatically.                  #
#       Add: Supports both weekly and monthly trials.                      #
#       Add: Added DIREXCLUDE.                                             #
############################################################################

ROOTFOLDER=/glftpd           # Path to glftpd's root dir
USERPATH=/ftp-data/users     # Path to users folder, relative to ROOTFOLDER.
GROUPFILE=/etc/group         # Full path to group file, relative to ROOTFOLDER.
PASSLIMIT=7500               # Amount, in MB, to pass trial.
DURATION=W                   # W = Weekly trial, M = Montly trial.

DIREXCLUDE='default.user|glftpd|backup|disabled'  # If you have folders in your
                                                  # users folder, add them here.
                                                  # If a user is part of the trial
                                                  # group, but his stats should not
                                                  # counted, add him here too.

###############################################################################
# No changes needed below here.                                            

if [ "$2" = "shell" ]; then
  USERPATH=$ROOTFOLDER$USERPATH
  GROUPFILE=$ROOTFOLDER$GROUPFILE
fi

## Dont change anything below here unless you know what you are doing.

## Do we want weekly or monthly trials?
case $DURATION in
 W)
   DUR="WKUP"
   DURNAME="week"
   ;;
 M)
   DUR="MONTHUP"
   DURNAME="month"
   ;;
 *)
   echo "Duration not set right. Either W or M"
   exit 0
   ;;
esac

if [ "$2" != "shell" ]; then
  if [ -z "$1" ]; then
    echo "Group Trial 0.5 by Turranius"
    echo "Current limit this $DURNAME is $PASSLIMIT MB"
    echo "Usage: gtrial <groupname>"
    echo "SiTEOPS: Set group nfo to TRiAL-2 for trial next $DURNAME."
    echo "         or TRiAL for trial this $DURNAME."
    echo "         Ex: site grpnfo SiTEOPS TRiAL-2"
    exit 0
  fi
else
  if [ -z "$1" ]; then
    echo "Usage: !gtrial <group>"
    exit 0
  fi
fi 

UPPED="0"
COUNT="0"
TOTAL="0"
EXISTS="NO"

cd $USERPATH

## Check if group exists
for i in `cat $GROUPFILE`
do
  NAME="$(echo $i | awk -F":" '{print $1}')"  
  if [ "$NAME" = "$1" ]; then
    EXISTS="Yes"
  fi
done

if [ "$EXISTS" != "Yes" ]; then
  echo "Group $1 does not exist."
  exit 0
fi

echo "-----[ Group Trial by Turranius ]-----"

## Check group users and what they upped
for i in `ls -f -A | /bin/egrep -v $DIREXCLUDE `
do
  GROUP="$(cat $i | grep -w GROUP | awk -F" " '{print $2}')"
  INIT="$( echo $GROUP | grep -w $1 )"
  if [ "$INIT" != "" ]; then
    UPPED="$(cat $i | grep -w $DUR | awk -F" " '{print $3}')"

    ## Check for other stat sections.
    UPPED2="$( grep -w $DUR $i | awk -F" " '{print $6}')"
    if [ "$UPPED2" != "" ]; then
      UPPED3="$(grep -w $DUR $i | awk -F" " '{print $9}')"
      if [ "$UPPED3" != "" ]; then
        UPPED4="$(grep -w $DUR $i | awk -F" " '{print $12}')"
        if [ "$UPPED4" != "" ]; then
          UPPED5="$(grep -w $DUR $i | awk -F" " '{print $15}')"
          if [ "$UPPED5" != "" ]; then
            UPPED6="$(grep -w $DUR $i | awk -F" " '{print $18}')"
            if [ "$UPPED6" != "" ]; then
              UPPED7="$(grep -w $DUR $i | awk -F" " '{print $21}')"
              if [ "$UPPED7" != "" ]; then
                UPPED8="$(grep -w $DUR $i | awk -F" " '{print $24}')"
                if [ "$UPPED8" != "" ]; then
                  UPPED9="$(grep -w $DUR $i | awk -F" " '{print $27}')"
                  if [ "$UPPED9" != "" ]; then
                    UPPED="$( expr "$UPPED" \+ "$UPPED2" \+ "$UPPED3" \+ "$UPPED4" \+ "$UPPED5" \+ "$UPPED6" \+ "$UPPED7" \+ "$UPPED8" \+ "$UPPED9" )"
                  else
                    UPPED="$( expr "$UPPED" \+ "$UPPED2" \+ "$UPPED3" \+ "$UPPED4" \+ "$UPPED5" \+ "$UPPED6" \+ "$UPPED7" \+ "$UPPED8" )"
                  fi
                else
                  UPPED="$( expr "$UPPED" \+ "$UPPED2" \+ "$UPPED3" \+ "$UPPED4" \+ "$UPPED5" \+ "$UPPED6" \+ "$UPPED7" )"
                fi
              else
                UPPED="$( expr "$UPPED" \+ "$UPPED2" \+ "$UPPED3" \+ "$UPPED4" \+ "$UPPED5" \+ "$UPPED6" )"
              fi
            else
              UPPED="$( expr "$UPPED" \+ "$UPPED2" \+ "$UPPED3" \+ "$UPPED4" \+ "$UPPED5" )"
            fi
          else
            UPPED="$( expr "$UPPED" \+ "$UPPED2" \+ "$UPPED3" \+ "$UPPED4" )"
          fi
        else
          UPPED="$( expr "$UPPED" \+ "$UPPED2" \+ "$UPPED3" )"
        fi
      else
        UPPED="$( expr "$UPPED" \+ "$UPPED2" )"
      fi
    fi

    UPPED="$(expr $UPPED \/ "1024")"
    echo "User: $i - $UPPED MB this $DURNAME."
    COUNT="$(expr $COUNT \+ 1)"
    TOTAL="$(expr $UPPED \+ $TOTAL)"
  fi
done

## Are they past the trial limit?
PASSED="$(expr $PASSLIMIT \- $TOTAL)"
if [ "$PASSED" -lt "1" ]; then
  OK="Yes"
else
  OK="No"
fi

echo "--------------------------------------"

## Read the trialgroup file to see if they are on trial or not.

NFO="$(cat $GROUPFILE | grep $1 | awk -F":" '{print $2}')"  
if [ "$NFO" = "TRiAL" ]; then
  GONTRIAL="YES"
else
  GONTRIAL="NO"
fi

if [ "$NFO" = "TRiAL-2" ]; then
  echo "Group $1 is up for trial next $DURNAME."
  GONTRIAL="YES"
  WEEK="NEXT"
fi

## Show stats for group.
echo "Total users: $COUNT"
echo "Limit to pass: "$PASSLIMIT". Upped: $TOTAL MB. Passed: $OK"

## Show how much they need to upload or how much they passed with.

if [ "$GONTRIAL" = "YES" ]; then
  if [ "$PASSED" -gt "0" ]; then
    if [ "$WEEK" = "NEXT" ]; then
      echo "$1 is up for trial next $DURNAME. Missing $PASSED MB to pass if it were this $DURNAME."
    else
      echo "$1 is in trial. Missing $PASSED MB to pass the trial."
    fi
  else
    if [ "$WEEK" = "NEXT" ]; then
      echo "$1 is up for trial next $DURNAME. Had it been this $DURNAME, they would have passed with "$PASSED"- MB"
    else
      echo "$1 has passed the trial with "$PASSED"- MB"
    fi
  fi
else
  if [ "$PASSED" -gt "0" ]; then
    echo "$1 is not on trial. Had they been, they are missing $PASSED MB"
  else
    echo "$1 is not on trial. Had they been, they would have passed with "$PASSED"- MB"  
  fi
fi

exit 0